﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using IWU = Infragistics.Win.UltraWinGrid;
using IW = Infragistics.Win;
using Infragistics.Win.UltraWinEditors;

namespace TestGrid
{
	public partial class Form1 : Form
	{
		private BindingList<DataItem> m_items;
		private string m_fakeValueColName = "FakeValueColumn";

		public Form1(BindingList<DataItem> items)
		{
			m_items = items;
			InitializeComponent();
			InitializeData();
			SetUI();
		}


		//create valuelists
		public void InitializeData()
		{

			this.ultraGrid1.DisplayLayout.ValueLists.Add("ItemType");
			string[] itemTypes = Enum.GetNames(typeof(ItemType));

			foreach (string item in itemTypes)
			{
				this.ultraGrid1.DisplayLayout.ValueLists["ItemType"].ValueListItems.Add(item);
			}


			foreach (ItemType type in Enum.GetValues(typeof(ItemType)))
			{
				string itemTypConditioneName = Enum.GetName(typeof(ItemType), type) + "Conditions";
				this.ultraGrid1.DisplayLayout.ValueLists.Add(itemTypConditioneName);
				List<string> conditions = ItemHelper.GetConditionSymbols(type);
				if (conditions != null)
				{
					foreach (string condition in conditions)
					{
						this.ultraGrid1.DisplayLayout.ValueLists[itemTypConditioneName].ValueListItems.Add(condition);
					}
				}

				string itemTypValueName = Enum.GetName(typeof(ItemType), type) + "Values";
				this.ultraGrid1.DisplayLayout.ValueLists.Add(itemTypValueName);
				List<string> values = ItemHelper.GetConditionValues(type);
				if (values!=null)
				{
					foreach (string value in values)
					{
						this.ultraGrid1.DisplayLayout.ValueLists[itemTypValueName].ValueListItems.Add(value);
					}
				}
			}
		}

		private void SetUI()
		{
			//disable column sort
			foreach (IWU.UltraGridColumn column in this.ultraGrid1.DisplayLayout.Bands[0].Columns)
			{
				column.SortIndicator = IWU.SortIndicator.Disabled;
			}
		}

		private IWU.ColumnsCollection Columns
		{
			get { return this.ultraGrid1.DisplayLayout.Bands[0].Columns; }
		}

		private IW.ValueListsCollection ValueLists
		{
			get { return this.ultraGrid1.DisplayLayout.ValueLists; }
		}


		private void Form1_Load(object sender, EventArgs e)
		{
			this.ultraGrid1.DisplayLayout.AutoFitStyle = IWU.AutoFitStyle.ResizeAllColumns;	
			this.ultraGrid1.DataSource = m_items;
		}

		

		private void ultraGrid1_BeforeCellActivate(object sender, IWU.CancelableCellEventArgs e)
		{
			try
			{
				if (e.Cell.Column.Key.Equals("Property"))
				{
					Columns["Property"].Style = IWU.ColumnStyle.DropDownList;

					e.Cell.ValueList = ValueLists["ItemType"];

				}
				else if (e.Cell.Column.Key.Equals("Condition"))
				{
					string typeName = e.Cell.Row.Cells["Property"].Value.ToString();

					if (string.IsNullOrEmpty(typeName)) return;

					string conditionName = typeName + "Conditions";

					Columns["Condition"].Style = IWU.ColumnStyle.DropDownList;

					e.Cell.ValueList = ValueLists[conditionName];

				}
				else if (e.Cell.Column.Key.Equals(m_fakeValueColName))
				{
					string typeName = e.Cell.Row.Cells["Property"].Value.ToString();

					if (string.IsNullOrEmpty(typeName)) return;

					ItemType type = (ItemType)Enum.Parse(typeof(ItemType), typeName);
					
					CtrlStyle ctrlType = ItemHelper.GetConditionCtrlStyle(type);

					if ( ctrlType != CtrlStyle.EditBox)
					{
						Columns[m_fakeValueColName].Style = IWU.ColumnStyle.DropDownList;

						if (ctrlType == CtrlStyle.DropDown)
						{
							e.Cell.ValueList = ValueLists[typeName + "Values"];
						}
						else
						{
							if (e.Cell.EditorComponent == null)
							{
								UltraComboEditor editor = new UltraComboEditor();
								editor.CheckedListSettings.CheckBoxStyle = Infragistics.Win.CheckStyle.CheckBox;
								editor.CheckedListSettings.ItemCheckArea = Infragistics.Win.ItemCheckArea.Item;
								editor.CheckedListSettings.EditorValueSource = Infragistics.Win.EditorWithComboValueSource.CheckedItems;
								editor.CheckedListSettings.ListSeparator = ItemHelper.Seperator;
								editor.ValueList = this.ultraGrid1.DisplayLayout.ValueLists[typeName + "Values"];
								e.Cell.EditorComponent = editor;
							}
						}
					}
				}
			}
			catch (NullReferenceException)
			{
				MessageBox.Show("Please make sure the frontal columns have been filled.");
			}

		}

		private void ultraGrid1_CellListSelect(object sender, IWU.CellEventArgs e)
		{

		}

	 
		private void BtnAdd_Click(object sender, EventArgs e)
		{
			m_items.AddNew();
		}

		private void BtnDel_Click(object sender, EventArgs e)
		{
			if (this.ultraGrid1.ActiveRow != null)
			{
				this.ultraGrid1.ActiveRow.Delete(false);
			}

		}

		private void ultraGrid1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
		{
			// override the backgroud color of the active cell. 
			this.ultraGrid1.DisplayLayout.Override.ActiveRowCellAppearance.BackColor = Color.White;
			this.ultraGrid1.DisplayLayout.Override.ActiveRowCellAppearance.ForeColor = Color.Black;

			this.Columns["Value"].Hidden = true;

			this.Columns.Add(m_fakeValueColName, "Value");
			this.Columns[m_fakeValueColName].DataType = typeof(List<string>);

		}

		private void ultraGrid1_BeforeRowUpdate(object sender, IWU.CancelableRowEventArgs e)
		{
			List<string> values = e.Row.Cells[m_fakeValueColName].Value as List<string>;

			if (values != null)
			{
				string newValue = string.Empty;
				foreach (string value in values)
				{
					newValue += value;
					if (values.IndexOf(value) < values.Count - 1)
					{
						newValue += ItemHelper.Seperator;
					}
				}

				m_items[e.Row.Index].Value = newValue;
			}
		}

		private void ultraGrid1_InitializeRow(object sender, IWU.InitializeRowEventArgs e)
		{
			if (e.ReInitialize == false)
			{
				string realValue = m_items[e.Row.Index].Value;
				if (!string.IsNullOrEmpty(realValue))
				{
					List<string> values = new List<string>(realValue.Split(ItemHelper.Seperator[0]));
					e.Row.Cells[m_fakeValueColName].Value = values;
				}
			}
		}
	}
}
